import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:simplecloudnotifier/components/badge_display/badge_display.dart';
import 'package:simplecloudnotifier/models/keytoken.dart';
import 'package:simplecloudnotifier/utils/toaster.dart';
import 'package:simplecloudnotifier/utils/ui.dart';

class KeyTokenCreatedModal extends StatelessWidget {
  final KeyToken keytoken;
  final String tokenValue;

  const KeyTokenCreatedModal({
    Key? key,
    required this.keytoken,
    required this.tokenValue,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      title: const Text('A new key was created'),
      content: Container(
        width: 0,
        height: 350,
        child: SingleChildScrollView(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.stretch,
            children: [
              UI.metaCard(
                context: context,
                icon: FontAwesomeIcons.solidIdCardClip,
                title: 'KeyTokenID',
                values: [keytoken.keytokenID],
              ),
              UI.metaCard(
                context: context,
                icon: FontAwesomeIcons.solidInputText,
                title: 'Name',
                values: [keytoken.name],
              ),
              UI.metaCard(
                context: context,
                icon: FontAwesomeIcons.solidShieldKeyhole,
                title: 'Permissions',
                values: _formatPermissions(keytoken.permissions),
              ),
              const SizedBox(height: 16),
              const BadgeDisplay(
                text: "Please copy and save the token now, it cannot be retrieved later.",
                icon: null,
                mode: BadgeMode.warn,
              ),
              const SizedBox(height: 4),
              UI.metaCard(
                context: context,
                icon: FontAwesomeIcons.solidKey,
                title: 'Token',
                values: [tokenValue.substring(0, 12) + '...'],
                iconActions: [(FontAwesomeIcons.copy, null, _copy)],
              ),
            ],
          ),
        ),
      ),
      actions: <Widget>[
        TextButton(
          child: const Text('Close'),
          onPressed: () {
            Navigator.of(context).pop();
          },
        ),
      ],
    );
  }

  List<String> _formatPermissions(String v) {
    var splt = v.split(';');

    if (splt.length == 0) return ["None"];

    List<String> result = [];

    if (splt.contains("A")) result.add("Admin");
    if (splt.contains("UR")) result.add("Read Account");
    if (splt.contains("CR")) result.add("Read Messages");
    if (splt.contains("CS")) result.add("Send Messages");

    return result;
  }

  void _copy() {
    Clipboard.setData(new ClipboardData(text: tokenValue));
    Toaster.info("Clipboard", 'Copied text to Clipboard');
    print('================= [CLIPBOARD] =================\n${tokenValue}\n================= [/CLIPBOARD] =================');
  }
}
