import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:provider/provider.dart';
import 'package:simplecloudnotifier/models/channel.dart';
import 'package:simplecloudnotifier/models/subscription.dart';
import 'package:simplecloudnotifier/models/user.dart';
import 'package:simplecloudnotifier/pages/subscription_view/subscription_view.dart';
import 'package:simplecloudnotifier/state/app_auth.dart';
import 'package:simplecloudnotifier/utils/navi.dart';

enum SubscriptionListItemMode {
  Messages,
  Extended,
}

class SubscriptionListItem extends StatelessWidget {
  const SubscriptionListItem({
    required this.item,
    required this.userCache,
    required this.channelCache,
    required this.needsReload,
    super.key,
  });

  final Subscription item;
  final Map<String, UserPreview> userCache;
  final Map<String, ChannelPreview> channelCache;

  final void Function()? needsReload;

  @override
  Widget build(BuildContext context) {
    final channelOwner = userCache[item.channelOwnerUserID];
    final subscriber = userCache[item.subscriberUserID];
    final channel = channelCache[item.channelID];

    return Card.filled(
      margin: EdgeInsets.fromLTRB(0, 4, 0, 4),
      shape: BeveledRectangleBorder(borderRadius: BorderRadius.circular(0)),
      color: Theme.of(context).cardTheme.color,
      child: InkWell(
        onTap: () {
          Navi.push(context, () => SubscriptionViewPage(subscriptionID: item.subscriptionID, preloadedData: (item, channelOwner, subscriber, channel), needsReload: this.needsReload));
        },
        child: Padding(
          padding: const EdgeInsets.all(8),
          child: Row(
            children: [
              Icon(FontAwesomeIcons.solidDiagramSubtask, color: Theme.of(context).colorScheme.outline, size: 32),
              SizedBox(width: 16),
              Expanded(
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.stretch,
                  children: [
                    Text(
                      subscriber?.username ?? item.subscriberUserID,
                      style: const TextStyle(),
                    ),
                    SizedBox(height: 4),
                    Row(
                      crossAxisAlignment: CrossAxisAlignment.end,
                      children: [
                        Text(
                          "@" + (channel?.displayName ?? item.channelID),
                          style: const TextStyle(fontWeight: FontWeight.bold),
                        ),
                        SizedBox(width: 10),
                        Text(
                          "(" + (channelOwner?.username ?? item.channelOwnerUserID) + ")",
                          style: const TextStyle(fontStyle: FontStyle.italic),
                        ),
                      ],
                    ),
                  ],
                ),
              ),
              SizedBox(width: 4),
              Padding(
                padding: const EdgeInsets.all(8),
                child: _buildIcon(context),
              ),
            ],
          ),
        ),
      ),
    );
  }

  Widget _buildIcon(BuildContext context) {
    final acc = Provider.of<AppAuth>(context, listen: false);

    final colorFull = Theme.of(context).colorScheme.onPrimaryContainer;
    final colorHalf = Theme.of(context).colorScheme.onPrimaryContainer.withAlpha(75);

    final isOwned = item.channelOwnerUserID == acc.userID && item.subscriberUserID == acc.userID;
    final isIncoming = item.channelOwnerUserID == acc.userID && item.subscriberUserID != acc.userID;
    final isOutgoing = item.channelOwnerUserID != acc.userID && item.subscriberUserID == acc.userID;

    if (isOutgoing && !item.confirmed) return Icon(FontAwesomeIcons.solidSquareEnvelope, color: colorHalf, size: 24);
    if (isOutgoing && !item.active) return Icon(FontAwesomeIcons.solidSquareShareNodes, color: colorHalf, size: 24);
    if (isOutgoing && item.active) return Icon(FontAwesomeIcons.solidSquareShareNodes, color: colorFull, size: 24);

    if (isIncoming && !item.confirmed) return Icon(FontAwesomeIcons.solidSquareQuestion, color: colorHalf, size: 24);
    if (isIncoming && !item.active) return Icon(FontAwesomeIcons.solidSquareCheck, color: colorHalf, size: 24);
    if (isIncoming && item.active) return Icon(FontAwesomeIcons.solidSquareCheck, color: colorFull, size: 24);

    if (isOwned && !item.confirmed) return Icon(FontAwesomeIcons.solidSquare, color: colorHalf, size: 24); // should not be possible
    if (isOwned && !item.active) return Icon(FontAwesomeIcons.solidSquareRss, color: colorHalf, size: 24);
    if (isOwned && item.active) return Icon(FontAwesomeIcons.solidSquareRss, color: colorFull, size: 24);

    return SizedBox(width: 24, height: 24); // should also not be possible
  }
}
